#include 'common.ch'
function duplikil( mast_dir, comp_dir, filemask )

local x, y, z, pointer
local comp_arry, mast_arry
local printit, killit, viewit, terse
local parmlist := upper( dosparam() )
local thisline
local work_arry := {}

default mast_dir to ''
default comp_dir to ''
default filemask to ''
printit := '/P' $ parmlist
killit  := '/D' $ parmlist
viewit  := '/V' $ parmlist
terse   := '/T' $ parmlist

if !killit .or. printit
   viewit := .t.
endif

if empty( mast_dir ) .or. empty( comp_dir ) .or. empty( filemask )
   ? 'DUPLIKIL.EXE'
   ? '   Used to analyze/remove identical files in a (group of) secondary directory'
   ? 'Usage:'
   ? '   DUPLIKIL <mast_dir> <@dirlist>/<comp_dir> <filemask> [/p] [/d] [/v]'
   ? '      Where:'
   ? '      <mast_dir> is the master directory including drive and path'
   ? '      <@dirlist> is a file containing a list of directories'
   ? '      <comp_dir> is the compare directory including drive and path'
   ? '      <filemask> is the dos file name(s) to operate on. ? and * are ok'
   ? '      [/p]       redirects analysis to the printer'
   ? '      [/v]       redirects analysis to video (default)'
   ? '      [/d]       deletes the identically duplicate files'
   ? '      [/t]       terse output. Only identical dupes show'
   ?
   ? '      -   Compared file was identical (and was deleted)'
   ? '      +   Compared file was different (and was not deleted)'
   ? '      ?   Compared file was not found in master directory'
   ?
   ? 'Examples:'
   ? '   DUPLIKIL c:\maindir @c:\maindir\dirlist.txt *.* /v'
   ? '   DUPLIKIL c:\maindir c:\maindir\temp *.* /p'
   quit
endif

if right( mast_dir, 1 ) == '\'
   mast_dir := left( mast_dir, len( mast_dir ) - 1 )
endif

if right( comp_dir, 1 ) == '\'
   comp_dir := left( comp_dir, len( comp_dir ) - 1 )
endif

mast_arry := directory( mast_dir + '\' + filemask )

if empty( mast_arry )
   ? filemask + '(<filemask>) was not found in master directory. Aborting'
   quit
endif

? comp_dir

if left( comp_dir, 1 ) == '@'
   if !bopen( substr( comp_dir, 2 ) )
      ? "Can't find List file" + substr( comp_dir, 2 )
      quit
   endif
   do while !beof()
      thisline := breadline()
      if !empty( thisline )
         aadd( work_arry, thisline )
      endif
   enddo
   bclose()
else
   work_arry := { comp_dir }
endif

do case
case viewit
   set printer to duplikil.$$$
   set print on
//   set console off
case printit
   set print on
endcase

for z := 1 to len( work_arry )

   comp_dir := work_arry[ z ]

   if right( comp_dir, 1 ) == '\'
      comp_dir := left( comp_dir, len( comp_dir ) - 1 )
   endif

   ?
   ? 'Comparing', mast_dir, 'to', comp_dir
   ?

   comp_arry := directory( comp_dir + '\' + filemask )

   if empty( comp_arry )
      ? filemask + '(<filemask>) was not found in compare directory'
   endif

   for x := 1 to len( comp_arry )
      pointer := ascan( mast_arry, { | y | y[ 1 ] == comp_arry[ x, 1 ] } )
      if pointer > 0
         if filecomp( mast_dir + '\' + mast_arry[ pointer, 1 ], comp_dir + '\' + comp_arry[ x, 1 ] )
            ? '- ' + comp_dir + '\' + comp_arry[ x, 1 ]
            if killit
               ferase( comp_dir + '\' + comp_arry[ x, 1 ] )
            endif
         else
            if !terse
               ? '+ ' + comp_dir + '\' + comp_arry[ x, 1 ]
            endif
         endif
      else
         if !terse
            ? '? ' + comp_dir + '\' + comp_arry[ x, 1 ]
         endif
      endif
   next
next

do case
case viewit
   set printer to
   set print off
   set console on
   fbrowse( 'duplikil.$$$' )
   ferase( 'duplikil.$$$' )
case printit
   set print off
   eject
endcase

return NIL

#define BUFFLEN  4096

function filecomp( mainfile, testfile )

local retval     := .f.      // default is that files do not compare

local buffer1    := space( BUFFLEN )
local bytesread1 := BUFFLEN
local handle1    := fopen( mainfile )
local buffer2    := space( BUFFLEN )
local bytesread2 := BUFFLEN
local handle2    := fopen( testfile )

if handle1 < 1 .or. handle2 < 1
   fclose( handle1 )
   fclose( handle2 )
   return retval
endif

retval := .t.            // now that they are open, default is true

do while bytesread1 == BUFFLEN

   bytesread1 := fread( handle1, @buffer1, BUFFLEN )

   if bytesread1 <> BUFFLEN
      buffer1 := left( buffer1, bytesread1 )
   endif

   bytesread2 := fread( handle2, @buffer2, BUFFLEN )

   if bytesread2 <> BUFFLEN
      buffer2 := left( buffer2, bytesread2 )
   endif

   if !( buffer1 == buffer2 )
      retval := .f.       // they failed synchronous comparison
      exit
   endif

enddo

fclose( handle1 )
fclose( handle2 )

return retval

